/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
const logUpdate = require("log-update");
const { observable, computed, action, decorate, autorun } = require("mobx");

class Timer {
  constructor() {
    this.start = Date.now();
    this.current = Date.now();
  }

  get elapsedTime() {
    return this.current - this.start + "milliseconds";
  }

  tick() {
    this.current = Date.now();
  }
}

decorate(Timer, {
  start: observable,
  current: observable,
  elapsedTime: computed,
  tick: action
});

const timer = new Timer();
autorun(() => {
  logUpdate(timer.elapsedTime);
});

const foo = () => {
  setTimeout(() => {
    timer.tick();
    foo();
  }, 1000);
};

foo();
